/* Copyright 2020, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "block_grid.hpp"
#include <QGridLayout>
#include <QHBoxLayout>
#include <QPainter>
#include <algorithm>

namespace duck::widget {
namespace {
	const constexpr int g_icon_spacing = 3;
} //unnamed namespace

BlockGrid::BlockGrid (QWidget* parent, unsigned int columns) :
	QWidget(parent),
	m_columns(columns)
{
}

BlockGrid::~BlockGrid() noexcept = default;

QSize BlockGrid::minimumSizeHint() const {
	return sizeHint();
}

QSize BlockGrid::sizeHint() const {
	if (m_columns) {
		const auto w = icon_width() * m_columns + g_icon_spacing * (1 + m_columns);
		const unsigned int rows = size() / m_columns;
		const auto h = icon_height() * rows + g_icon_spacing * (rows + 1);

		return {static_cast<int>(w), static_cast<int>(h)};
	}
	else {
		return {-1, -1};
	}
}

unsigned int BlockGrid::icon_width() const {
	return m_icons.empty() ? static_cast<unsigned int>(g_icon_spacing) : m_icons.front().size().width();
}

unsigned int BlockGrid::icon_height() const {
	return m_icons.empty() ? static_cast<unsigned int>(g_icon_spacing) : m_icons.front().size().height();
}

void BlockGrid::push_back (AnimatedPixmap&& anim) {
	m_icons.push_back(std::move(anim));
}

std::size_t BlockGrid::size() const {
	return m_icons.size();
}

unsigned int BlockGrid::columns() const {
	if (m_columns) {
		return m_columns;
	}
	else {
		const auto widget_w = static_cast<unsigned int>(std::max(this->width(), g_icon_spacing));
		const auto ico_spacing_uns = static_cast<unsigned int>(g_icon_spacing);

		return (widget_w - ico_spacing_uns) / (icon_width() + ico_spacing_uns);
	}
}

void BlockGrid::paintEvent (QPaintEvent* event) {
	QPainter painter(this);
	const int max_col = static_cast<int>(this->columns());
	int x = 0;
	int y = 0;
	int count = 0;

	for (auto& ico : m_icons) {
		x = g_icon_spacing + (count % max_col) * (g_icon_spacing + ico.size().width());
		y = g_icon_spacing + (count / max_col) * (g_icon_spacing + ico.size().height());
		if (ico.frame_count()) {
			painter.drawPixmap(x, y, ico.current_frame());
		}

		++count;
	}
}
} //namespace duck::widget
